#!/usr/bin/env python3
#
# Author: Joe Block <jblock@zscaler.com>
# License: Apache 2.0
# Copyright 2022, ZScaler Inc.
#
# Interact with JIRA

import getpass
import json
import logging
import requests
from requests.auth import HTTPBasicAuth

from jira import JIRA
from jira_commands.utils import dumpObject
from thelogrus.yaml import readYamlFile


def loadJiraSettings(path: str, cli):
    """
    Load JIRA settings from a yaml file, allowing overrides from the CLI
    """
    settings = readYamlFile(path=path)
    # Command line arguments should override the settings file
    if cli.server:
        settings["jira_server"] = cli.server

    if cli.username:
        settings["username"] = cli.username

    if cli.password:
        settings["password"] = cli.password

    if "username" not in settings:
        settings["username"] = input("Username: ")

    if "password" not in settings:
        settings["password"] = getpass.getpass("Password: ")

    # Make sure we have all the settings we need
    if "jira_server" not in settings:
        raise RuntimeError("You must specify the jira server")
    if not settings["username"]:
        raise RuntimeError("You must specify the jira server username")
    if not settings["password"]:
        raise RuntimeError("You must specify the jira server password")

    logging.debug(f"Using JIRA server: {settings['jira_server']}")
    logging.debug(f"username: {settings['username']}")

    credentials = {"username": settings["username"], "password": settings["password"]}
    if "credentials" not in settings:
        logging.debug("Setting credentials key in settings")
        settings["credentials"] = credentials
    else:
        logging.warning(f"There is already a credentials key in {path}")
    return settings


def makeIssueData(cli):
    """
    Create issue_data from command line arguments

    This sets up the standard issue data fields - if a JIRA project has
    custom fields, it should get it's own issueData function that starts
    by calling this.

    returns dict
    """
    try:
        if cli.json:
            issue_data = json.loads(cli.json)
            logging.debug(f"issue_data (from --json): {issue_data}")
        else:
            logging.debug("Starting with blank issue data")
            issue_data = {}
    except AttributeError:
        logging.warning("No json command line argument found")

    if cli.description:
        logging.debug(f"description: {cli.description}")
        issue_data["description"] = cli.description

    if cli.issue_type:
        logging.debug(f"issue_type: {cli.issue_type}")
        issue_data["issuetype"] = cli.issue_type

    if cli.label:
        logging.debug(f"label: {cli.label}")
        issue_data["label"] = cli.label

    if cli.project:
        logging.debug(f"project: {cli.project}")
        issue_data["project"] = cli.project

    if cli.summary:
        logging.debug(f"summary: {cli.summary}")
        issue_data["summary"] = cli.summary

    return issue_data


def linkIssuesHack(
    source: str,
    target: str,
    link_type: str,
    username: str,
    password: str,
    jira_server: str,
):
    """
    Link two issues

    This is a horrible hack because the jira module fails with a permission
    error when I use its create_issue_link method, but I can use the same
    username and password with curl against the JIRA API directly and that
    works, so created an issue upstream and I'm using this requests.get hack
    until https://github.com/pycontribs/jira/issues/1296 is fixed upstream.

    Based on https://confluence.atlassian.com/jirakb/how-to-use-rest-api-to-add-issue-links-in-jira-issues-939932271.html
    """

    # This is documented to work, but returns an error that we don't have
    # link issue permission.

    # logging.info(f"Creating '{link_type}' link from {source} to {target}")
    # result = self.connection.create_issue_link(
    #     type=link_type, inwardIssue=source, outwardIssue=target
    # )

    data = {
        "update": {
            "issuelinks": [
                {
                    "add": {
                        "type": {
                            "name": link_type,
                        },
                        "outwardIssue": {"key": target},
                    }
                }
            ]
        }
    }
    url = f"{jira_server}/rest/api/2/issue/{source}"

    logging.debug(f"username: {username}")
    logging.debug(f"url: {url}")
    logging.debug(f"data: {data}")

    results = requests.put(url, auth=HTTPBasicAuth(username, password), json=data)
    logging.debug(f"status code: {results.status_code}")
    if results.status_code >= 200 and results.status_code < 300:
        logging.debug("Successful")
        logging.debug(f"results: {results}")
        status = True
    else:
        logging.error(f"Call failed: {results.status_code}")
        logging.error(f"results: {results}")
        status = False
    return status


class JiraTool:
    # Jira housekeeping
    def __init__(self, settings: dict):
        """
        Create a JIRA helper object
        """
        self.username = settings["username"]
        self.password = settings["password"]
        self.jira_server = settings["jira_server"]
        self.connect()

    def __str__(self):
        """
        Print a representation of the object
        """
        raw = {"username": self.username, "jira_server": self.jira_server}
        return raw.__str__()

    def connect(self):
        jiraOptions = {"server": self.jira_server}
        jiraBasicAuth = (self.username, self.password)
        logging.debug(
            f"Creating connection to {self.jira_server} with user {self.username}"
        )
        self.connection = JIRA(options=jiraOptions, basic_auth=jiraBasicAuth)

    # Utility functions
    def assignTicket(self, ticket: str, assignee: str):
        """
        Assign a ticket
        """
        logging.debug(f"Assigning {ticket} to {assignee}")
        return self.connection.assign_issue(ticket, assignee)

    def unassignTicket(self, ticket: str):
        """
        Assign a ticket to no one
        """
        logging.debug(f"Assigning {ticket} to No one")
        return self.connection.assign_issue(ticket, None)

    def addComment(self, ticket: str, comment: str):
        """
        Comment on a ticket
        """
        if comment:
            logging.debug(f"Adding comment {comment} to ticket {ticket}")
            return self.connection.add_comment(ticket, comment)
        else:
            raise RuntimeError("You must specify a comment to add to the ticket")

    def allowedValuesForField(self, ticket: str, custom_field: str):
        """
        Get the allowed values for a ticket custom field

        JIRA isn't very forgiving about ticket values, so provide a way to
        extract what it's expecting to find in a given custom field.
        """
        logging.debug(f"connection: {self.connection}")

        issue = self.getIssueData(ticket)
        logging.debug(f"issue: {issue}")

        meta = self.getIssueMetaData(ticket=ticket)
        allowed = meta["fields"][custom_field]["allowedValues"]
        return allowed

    def createTicket(self, issue_data: dict, priority: str = None, strict=True):
        """
        Create a JIRA ticket from a data dictionary
        """
        logging.debug(f"Creating ticket using {issue_data}")
        # Make sure we have a minimum set of fields
        required = [
            "description",
            "summary",
            "project",
            "issuetype",
        ]
        if strict:
            valid = True
            for r in required:
                if r not in issue_data:
                    valid = False
                    logging.error(f"{r} not specified in issue_data")
            if not valid:
                logging.critical(
                    f"You must specify all the mandatory issue fields: {required}"
                )
                raise ValueError(
                    f"You must specify all the mandatory issue fields: {required}"
                )

        if priority:
            logging.debug(f"Setting ticket priority to {priority}")
            priority_info = self.getPriorityDict()
            priority_data = {"id": priority_info[priority]}
            issue_data["priority"] = priority_data

        new_issue = self.connection.create_issue(fields=issue_data)
        return new_issue

    def createSubtask(self, issue_data: dict, parent: str):
        """
        Create a subtask
        """
        logging.warning("Creating a subtask")
        if not parent:
            logging.error("You must specify a parent ticket when creating a Sub-Task")
            raise ValueError(
                "You must specify a parent ticket when creating a Sub-Task"
            )
        issue_data["parent"] = {"id": parent}
        return self.createTicket(issue_data=issue_data)

    def getIssueData(self, ticket: str):
        """
        Returns the JIRA issue data for a ticket

        This is a shim to keep JiraTool users from having to rummage through
        its internals to use the jira object it's connecting to your jira
        server with.

        Args:
            ticket (str): JIRA ticket number
        """
        return self.connection.issue(ticket)

    def getIssueMetaData(self, ticket: str):
        """
        Get an issue's metadata.

        This is a shim to keep JiraTool users from having to rummage through
        its internals to use the jira object it's connecting to your jira
        server with.

        Args:
            ticket (str): JIRA ticket number
        """
        issue = self.getIssueData(ticket=ticket)
        meta = self.connection.editmeta(issue)
        return meta

    def linkIssues(self, source, target, link_type):
        """
        Link two issues
        """
        # Jira is inconsistent about when you can use string ticket ids and
        # when you have to use issue objects
        source_issue = self.connection.issue(source)
        target_issue = self.connection.issue(target)
        logging.debug(f"source_issue: {source_issue}")
        logging.debug(f"target_issue: {target_issue}")

        # Link the two issues
        return linkIssuesHack(
            username=self.username,
            password=self.password,
            link_type=link_type,
            source=source,
            target=target,
            jira_server=self.jira_server,
        )

    def listTickets(self, project: str):
        for singleIssue in self.connection.search_issues(
            jql_str=f"project = {project}"
        ):
            print(
                f"{singleIssue.key} {singleIssue.fields.summary} {singleIssue.fields.reporter.displayName}"
            )

    def getPriorityDict(self):
        """
        Returns a dictionary of all the priorities on a server and their IDs
        """
        raw_priorities = self.connection.priorities()
        priority_data = {}

        for priority in raw_priorities:
            logging.debug(f"{priority.name} : {priority.id}")
            priority_data[priority.name] = priority.id
        return priority_data

    def getTicket(self, ticket: str):
        """
        Peel a ticket out of JIRA
        """
        issue = self.connection.issue(ticket)
        return issue

    def vivisect(self, ticket_id: str):
        """
        Vivisect a ticket so we can figure out what attributes are visible
        via the module's API.
        """
        ticket = self.getTicket(ticket=ticket_id)
        print(f"ticket: {ticket}")
        print("ticket transitions available:")
        for transition in self.connection.transitions(ticket):
            print(f"  {transition}")
        print(f"ticket.fields.issuetype: {ticket.fields.issuetype}")
        print(f"ticket.fields.issuelinks: {ticket.fields.issuelinks}")
        print(f"ticket.fields.issuelinks dump: {dumpObject(ticket.fields.issuelinks)}")
        print(f"ticket.fields: {ticket.fields}")
        print()
        print(f"ticket.fields (dump): {dumpObject(ticket.fields)}")

    def getTicketDict(self, project: str):
        """
        Get JIRA tickets in a project, return as a dict
        """
        tickets = {}
        for singleIssue in self.connection.search_issues(
            jql_str=f"project = {project}"
        ):
            tickets[singleIssue.key] = singleIssue
            logging.debug(f"{singleIssue.key} : {singleIssue}")
            logging.debug(f"{singleIssue.key} : fields {singleIssue.fields}")
            logging.debug(f"dumpObj(singleIssue : {dumpObject(singleIssue)}")
            logging.debug(" ")
        return tickets

    def transitionTicket(self, ticket: str, state: str, comment: str = None):
        """
        Transition a ticket to a new state
        """
        issue = self.connection.issue(ticket)
        available_transitions = self.ticketTransitions(ticket=ticket)

        if state in available_transitions:
            logging.info(f"Transitioning issue {ticket} to state {state}")
            if comment:
                self.addComment(ticket=ticket, comment=comment)
            return self.connection.transition_issue(issue, available_transitions[state])
        else:
            raise ValueError(
                f"{ticket} does not have {state} as an available transition. Perhaps your user doesn't have privilege for that?"
            )

    # Internal helpers
    def ticketTransitions(self, ticket: str):
        """
        Find the available transitions for a given ticket
        """

        # Map the names to ids so the caller can user a human-understandable
        # name instead of having to track down the id.
        transitions = {}
        for t in self.connection.transitions(ticket):
            logging.debug(f"Found transition '{t['name']}, id {t['id']}")
            transitions[t["name"]] = t["id"]
        logging.debug(f"Transition lookup table: {transitions}")
        return transitions



#!/usr/bin/env python3
#
# interact with jira
#
# Author: Joe Block <jblock@zscaler.com>
# License: Apache 2.0
# Copyright 2022, ZScaler Inc.

import json
import logging
import sys

from jira_commands.cli.common import baseCLIParser, parseTicketCLI, ticketCreationParser
from jira_commands.jira import JiraTool, loadJiraSettings, makeIssueData


def parseTicketAssignCLI():
    """
    Command line options for assigning a ticket
    """
    parser = parseTicketCLI(description="Assign a JIRA ticket to someone")
    parser.add_argument(
        "--assignee",
        type=str,
        required=True,
        help="Username to assign ticket to. Specify None if you want to unassign the ticket",
    )
    cli = parser.parse_args()
    loglevel = getattr(logging, cli.log_level.upper(), None)
    logFormat = "[%(asctime)s][%(levelname)8s][%(filename)s:%(lineno)s - %(funcName)20s() ] %(message)s"
    logging.basicConfig(level=loglevel, format=logFormat)
    logging.info("Set log level to %s", cli.log_level.upper())
    return cli


def parseTicketCommentCLI():
    """
    Command line options for commmenting on a ticket
    """
    parser = parseTicketCLI(description="Comment on a JIRA ticket")
    parser.add_argument(
        "--comment",
        type=str,
        required=True,
        help="Comment to add to the specified ticket, It only supports very limited formatting - _italic_ and *bold* work, but `code` doesn't.",
    )
    cli = parser.parse_args()
    loglevel = getattr(logging, cli.log_level.upper(), None)
    logFormat = "[%(asctime)s][%(levelname)8s][%(filename)s:%(lineno)s - %(funcName)20s() ] %(message)s"
    logging.basicConfig(level=loglevel, format=logFormat)
    logging.info("Set log level to %s", cli.log_level.upper())
    return cli


def parseTicketCloseCLI():
    """
    Command line options for closing a ticket
    """
    parser = parseTicketCLI(description="Close a JIRA ticket")
    parser.add_argument(
        "--comment",
        type=str,
        help="Comment to add to the specified ticket, It only supports very limited formatting - _italic_ and *bold* work, but `code` doesn't.",
    )
    cli = parser.parse_args()
    loglevel = getattr(logging, cli.log_level.upper(), None)
    logFormat = "[%(asctime)s][%(levelname)8s][%(filename)s:%(lineno)s - %(funcName)20s() ] %(message)s"
    logging.basicConfig(level=loglevel, format=logFormat)
    logging.info("Set log level to %s", cli.log_level.upper())
    return cli


def parseTicketInspectionCLI():
    """
    Command line options for ticket inspectors
    """
    parser = parseTicketCLI(
        description="Vivisect a JIRA ticket so we can determine which custom fields map to which data keys"
    )
    cli = parser.parse_args()
    loglevel = getattr(logging, cli.log_level.upper(), None)
    logFormat = "[%(asctime)s][%(levelname)8s][%(filename)s:%(lineno)s - %(funcName)20s() ] %(message)s"
    logging.basicConfig(level=loglevel, format=logFormat)
    logging.info("Set log level to %s", cli.log_level.upper())
    return cli


def parseCreateTicketCLI():
    """
    Parse the command line options
    """
    parser = ticketCreationParser(description="Create a JIRA ticket")
    cli = parser.parse_args()

    loglevel = getattr(logging, cli.log_level.upper(), None)
    logFormat = "[%(asctime)s][%(levelname)8s][%(filename)s:%(lineno)s - %(funcName)20s() ] %(message)s"
    logging.basicConfig(level=loglevel, format=logFormat)
    logging.info("Set log level to %s", cli.log_level.upper())

    # Validity checks
    if cli.issue_type == "Sub-task":
        if not cli.parent:
            logging.error(
                "You must specify a parent with --parent when you are creating a subtask"
            )
            sys.exit(13)
    return cli


def parseGetTransitionsCLI():
    """
    Parse the command line options for transition list tool
    """
    parser = parseTicketCLI(
        description="See all transitions available on a JIRA ticket"
    )

    cliArgs = parser.parse_args()
    loglevel = getattr(logging, cliArgs.log_level.upper(), None)
    logFormat = "[%(asctime)s][%(levelname)8s][%(filename)s:%(lineno)s - %(funcName)20s() ] %(message)s"
    logging.basicConfig(level=loglevel, format=logFormat)
    logging.debug("Set log level to %s", cliArgs.log_level.upper())
    return cliArgs


def parseTicketLinkCLI():
    """
    Command line options for linking two tickets
    """
    parser = parseTicketCLI(description="Link two JIRA tickets")
    parser.add_argument(
        "--target",
        type=str,
        required=True,
        help="Target ticket",
    )

    link_types = [
        "Blocks",
        "Depends",
        "Bugs" "Clones",
    ]
    parser.add_argument(
        "--link-type",
        type=str,
        required=True,
        help=f"Link type. Case matters. Consider {link_types} as options, "
        "though your server may have other types too. 'jc get link types' "
        "will show all the link types on your JIRA server",
    )
    cli = parser.parse_args()
    loglevel = getattr(logging, cli.log_level.upper(), None)
    logFormat = "[%(asctime)s][%(levelname)8s][%(filename)s:%(lineno)s - %(funcName)20s() ] %(message)s"
    logging.basicConfig(level=loglevel, format=logFormat)
    logging.debug("Set log level to %s", cli.log_level.upper())
    return cli


def parseTransitionToCLI():
    """
    Parse the command line options for transition set tool
    """
    parser = parseTicketCLI(
        description="See all transitions available on a JIRA ticket"
    )
    parser.add_argument(
        "--transition-to", help="Transition a ticket to a named state", type=str
    )

    cliArgs = parser.parse_args()
    loglevel = getattr(logging, cliArgs.log_level.upper(), None)
    logFormat = "[%(asctime)s][%(levelname)8s][%(filename)s:%(lineno)s - %(funcName)20s() ] %(message)s"
    logging.basicConfig(level=loglevel, format=logFormat)
    logging.debug("Set log level to %s", cliArgs.log_level.upper())
    return cliArgs


def assignTicket():
    """
    Assign a ticket to someone
    """
    cli = parseTicketAssignCLI()
    logging.debug(f"cli: {cli}")

    settings = loadJiraSettings(path=cli.settings_file, cli=cli)

    jira = JiraTool(settings=settings)
    if cli.assignee.lower() == "none":
        jira.unassignTicket(ticket=cli.ticket)
    else:
        jira.assignTicket(ticket=cli.ticket, assignee=cli.assignee)


def commentOnTicket():
    """
    Comment on a ticket
    """
    cli = parseTicketCommentCLI()
    logging.debug(f"cli: {cli}")

    settings = loadJiraSettings(path=cli.settings_file, cli=cli)

    jira = JiraTool(settings=settings)
    jira.addComment(ticket=cli.ticket, comment=cli.comment)


def closeTicket():
    """
    Close a ticket
    """
    cli = parseTicketCommentCLI()
    logging.debug(f"cli: {cli}")

    settings = loadJiraSettings(path=cli.settings_file, cli=cli)

    jira = JiraTool(settings=settings)
    jira.transitionTicket(ticket=cli.ticket, state="Done", comment=cli.comment)


def createTicket():
    """
    Create a JIRA ticket
    """
    cli = parseCreateTicketCLI()
    logging.debug(f"cli: {cli}")

    settings = loadJiraSettings(path=cli.settings_file, cli=cli)
    issue_data = makeIssueData(cli=cli)

    jira = JiraTool(settings=settings)
    if cli.issue_type == "Sub-task":
        results = jira.createSubtask(issue_data=issue_data, parent=cli.parent)
    else:
        results = jira.createTicket(
            issue_data=issue_data, strict=False, priority=cli.priority
        )
    print(results)
    # return results


def getLinkTypes():
    """
    Get all the link types on a server
    """
    parser = baseCLIParser()
    parser.add_argument("--json", help="Output in JSON format", action="store_true")
    cli = parser.parse_args()

    loglevel = getattr(logging, cli.log_level.upper(), None)
    logFormat = "[%(asctime)s][%(levelname)8s][%(filename)s:%(lineno)s - %(funcName)20s() ] %(message)s"
    logging.basicConfig(level=loglevel, format=logFormat)
    logging.debug("Set log level to %s", cli.log_level.upper())

    settings = loadJiraSettings(path=cli.settings_file, cli=cli)

    jira = JiraTool(settings=settings)

    link_type_names = []
    for link_type in jira.connection.issue_link_types():
        logging.debug(link_type.name)
        link_type_names.append(link_type.name)
    if cli.json:
        print(json.dumps({"link_types": link_type_names}, indent=2))
    else:
        print(f"Link type names: {link_type_names}")


def linkTickets():
    """
    Link two tickets
    """
    cli = parseTicketLinkCLI()
    logging.debug(f"cli: {cli}")

    settings = loadJiraSettings(path=cli.settings_file, cli=cli)

    jira = JiraTool(settings=settings)
    results = jira.linkIssues(
        source=cli.ticket, target=cli.target, link_type=cli.link_type
    )
    logging.debug(results)
    if results:
        print(f"({cli.link_type}) link created between {cli.ticket} and {cli.target}")
    else:
        print(
            f"Could not create ({cli.link_type})Link between {cli.ticket} and {cli.target}"
        )
    print(results)


def getPriorities():
    """
    Get all the priorities on a server
    """
    parser = baseCLIParser(
        description="Get list of priorities on a server and their IDs"
    )
    parser.add_argument("--json", help="Output in JSON format", action="store_true")
    cli = parser.parse_args()

    loglevel = getattr(logging, cli.log_level.upper(), None)
    logFormat = "[%(asctime)s][%(levelname)8s][%(filename)s:%(lineno)s - %(funcName)20s() ] %(message)s"
    logging.basicConfig(level=loglevel, format=logFormat)
    logging.debug("Set log level to %s", cli.log_level.upper())

    settings = loadJiraSettings(path=cli.settings_file, cli=cli)

    jira = JiraTool(settings=settings)
    priority_data = jira.getPriorityDict()

    if cli.json:
        print(json.dumps({"priorities": priority_data}, indent=2))
    else:
        print(f"Issue Priorities: {priority_data}")


def getTransitions():
    """
    Print all the available transitions on a given ticket
    """
    cli = parseGetTransitionsCLI()
    logging.debug(f"cli: {cli}")

    settings = loadJiraSettings(path=cli.settings_file, cli=cli)

    jira = JiraTool(settings=settings)
    print(jira.ticketTransitions(ticket=cli.ticket))


def transitionTo():
    """
    Transition a given ticket to a specified state
    """
    cli = parseTransitionToCLI()
    logging.debug(f"cli: {cli}")

    settings = loadJiraSettings(path=cli.settings_file, cli=cli)

    jira = JiraTool(settings=settings)
    print(jira.transitionTicket(ticket=cli.ticket, state=cli.transition_to))


if __name__ == "__main__":
    raise RuntimeError("This file should not be run directly, import functions from it")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['jira_commands', 'jira_commands.cli']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0', 'jira>=3.1.1,<4.0.0', 'thelogrus>=0.6.2,<0.7.0']

entry_points = \
{'console_scripts': ['jc = jira_commands.cli.jc:jc_driver',
                     'jc-assign-ticket = '
                     'jira_commands.cli.crudops:assignTicket',
                     'jc-close-ticket = jira_commands.cli.crudops:closeTicket',
                     'jc-comment-on-ticket = '
                     'jira_commands.cli.crudops:commentOnTicket',
                     'jc-create-ticket = '
                     'jira_commands.cli.crudops:createTicket',
                     'jc-custom-field-allowed-values = '
                     'jira_commands.cli.vivisect:listAllowedFieldValues',
                     'jc-examine-ticket = jira_commands.cli.vivisect:vivisect',
                     'jc-get-link-types = '
                     'jira_commands.cli.crudops:getLinkTypes',
                     'jc-get-priorities = '
                     'jira_commands.cli.crudops:getPriorities',
                     'jc-get-priority-ids = '
                     'jira_commands.cli.crudops:getPriorities',
                     'jc-link-tickets = jira_commands.cli.crudops:linkTickets',
                     'jc-list-project-tickets = '
                     'jira_commands.cli.list:listTickets',
                     'jc-list-ticket-transitions = '
                     'jira_commands.cli.crudops:getTransitions',
                     'jc-ticket-assign = '
                     'jira_commands.cli.crudops:assignTicket',
                     'jc-ticket-close = jira_commands.cli.crudops:closeTicket',
                     'jc-ticket-comment = '
                     'jira_commands.cli.crudops:commentOnTicket',
                     'jc-ticket-create = '
                     'jira_commands.cli.crudops:createTicket',
                     'jc-ticket-examine = jira_commands.cli.vivisect:vivisect',
                     'jc-ticket-link = jira_commands.cli.crudops:linkTickets',
                     'jc-ticket-transition-list = '
                     'jira_commands.cli.crudops:getTransitions',
                     'jc-ticket-transition-set = '
                     'jira_commands.cli.crudops:transitionTo',
                     'jc-ticket-vivisect = jira_commands.cli.vivisect:vivisect',
                     'jc-transition-ticket-to = '
                     'jira_commands.cli.crudops:transitionTo',
                     'jc-vivisect-ticket = '
                     'jira_commands.cli.vivisect:vivisect']}

setup_kwargs = {
    'name': 'jira-commands',
    'version': '0.5.0',
    'description': 'Command line utilities for interacting with JIRA',
    'long_description': "# jira-commands\n\n[![License](https://img.shields.io/badge/License-Apache%202.0-blue.svg)](https://opensource.org/licenses/Apache-2.0)\n[![Build Status](https://img.shields.io/endpoint.svg?url=https%3A%2F%2Factions-badge.atrox.dev%2Funixorn%2Fjira-commands%2Fbadge%3Fref%3Dmain&style=plastic)](https://actions-badge.atrox.dev/unixorn/jira-commands/goto?ref=main)\n![Megalinter](https://github.com/unixorn/jira-commands/actions/workflows/mega-linter.yml/badge.svg)\n\nSome command-line tools for interacting with JIRA.\n\n## Scripts\n\nAll of these scripts support `--help` to get a detailed list of command line options.\n\n| Name                         | Description                                           |\n| -----------------------------| ----------------------------------------------------- |\n| `jc` | Main driver. Will run all the other commands inside a docker container for you. |\n| `jc assign ticket` / `jc ticket assign` | Assign a ticket to someone. |\n| `jc close ticket` / `jc ticket close` | Close a ticket |\n| `jc comment on ticket` / `jc ticket comment` | Comment on a ticket |\n| `jc create ticket` / `jc ticket create` | Create a ticket. You will need|\n| `jc custom field allowed values` | List a custom field's allowed values since JIRA isn't forthcoming about them. |\n| `jc examine ticket` / `jc ticket examine` | Detailed dump of a ticket and all its custom field names |\n| `jc get link types` | Prints the names of all link types defined on your JIRA instance. |\n| `jc get priority ids` | Prints the names of all ticket priorities defined on your JIRA instance. |\n| `jc link tickets` / `jc ticket link` | Link two tickets. Use `jc get link types` to see what link names are defined on your JIRA server. Case matters. |\n| `jc list project tickets` | List open tickets in a given JIRA project |\n| `jc list ticket transitions` / `jc ticket transition list` | See the availale transitions for a given ticket. |\n| `jc transition ticket to` / `jc ticket transition set` | Transition a ticket to another state. Use `jc list ticket transitions` to see which are available  |\n| `jc vivisect ticket` / `jc ticket vivisect` | Detailed dump of a ticket to find out all the custom field names and other innards. |\n\nThe `jc` program is the main driver script and will find the subcommands, so you can do `jc ticket comment --ticket ABC-123 --comment 'foo bar baz'` and it will find the `jc-ticket-comment` script and run it with the `--ticket` and `--comment` arguments.\n\nIf you're using the docker method, `jc` will automatically run the subcommands inside a container for you. If you've installed via pip, it'll find the commands where they were installed in your `$PATH`.\n\n## Configuration\n\nThe `jc` commands all read settings from `~/.jira-commands/jira.yaml`. Settings in the file can be overridden by specifying command-line options.\n\nI'm setting my username and jira server in the example below. The tools will ask for my password when I run them.\n\n```yaml\nusername: yourusername\njira_server: https://jira.example.com\n```\n\nYou can specify a `password` key but it's a terrible idea.\n\n## Installation\n\n### Run via docker / nerdctl\n\nThis is the recommended way to use the `jc` commands, and how it will be run if you use one of the ZSH frameworks detailed below.\n\nIf you're not using a ZSH framework, clone this repository and add its `bin` directory to your `$PATH`. It contains a `jc` script that will detect whether you have `nerdctl` or `docker` and if it finds them, map `~/jira-commands` (and the configuration file there) into a volume in the `jira-commands` container and run the tools inside the container.\n\n### Direct pip install\n\n`sudo pip install jira-commands` will install the command-line tools via `pip`. This may cause compatibility annoyances with other python tools on your system, so there's a `docker`/`nerdctl` option as well.\n\n### ZSH plugin\n\nThe tooling has been packaged as a ZSH plugin to make using it as easy as possible for ZSH users.\n\n#### zgenom\n\nIf you're using [Zgenom](https://github.com/jandamm/zgenom):\n\n1. Add `zgenom load unixorn/jira-commands` to your `.zshrc` with your other plugins\n2. `zgenom reset && zgenom save`\n\n#### Antigen\n\nIf you're using [Antigen](https://github.com/zsh-users/antigen):\n\n1. Add `antigen bundle unixorn/jira-commands` to your .zshrc where you've listed your other plugins.\n2. Close and reopen your Terminal/iTerm window to refresh context and use the plugin. Alternatively, you can run `antigen bundle unixorn/jira-commands` in a running shell to have `antigen` load the new plugin.\n\n#### oh-my-zsh\n\nIf you're using [oh-my-zsh](https://ohmyz.sh):\n\n1. Clone the repository into a new `jira-commands` directory in oh-my-zsh's plugin folder:\n\n    `git clone https://github.com/unixorn/jira-commands.git $ZSH_CUSTOM/plugins/jira-commands`\n\n2. Edit your `~/.zshrc` and add `jira-commands` – same as clone directory – to the list of plugins to enable:\n\n    `plugins=( ... jira-commands )`\n\n3. Then, restart your terminal application to refresh context and use the plugin. Alternatively, you can source your current shell configuration:\n\n    `source ~/.zshrc`\n",
    'author': 'Joe Block',
    'author_email': 'jblock@zscaler.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/unixorn/jira-commands',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
